
package src;

import java.util.ArrayList;

public class AcSignal{
	private String name;
	private ArrayList<ReceiveSignalEvent> registeredEvents;	//List for the ReceiveSignalEvents which must be noticed when signal was sent
	
	public AcSignal(String _name){
		name = _name;
		registeredEvents = new ArrayList<ReceiveSignalEvent>();
	}
	
	public String getName(){
		return name;
	}
	
	public void setName(String _name){
		name = _name;
	}
	
	//Method to register new ReceiveSignalEvents
	public void registerEvent(ReceiveSignalEvent ev){
		if(!registeredEvents.contains(ev))
			registeredEvents.add(ev);
	}
	
	//Method which is called by a SendSignalAction which sends this signal. The registered RecSigEvents must be noticed
	public void send(){
		System.out.println("Signal " + name + " sent");
		for(int i = 0; i<= registeredEvents.size()-1; i++)
			registeredEvents.get(i).setTrigger(true);
	}
}

abstract class Event{
	protected String id = "";
	
	public String getId(){
		return "";
	}
	
	public abstract void setTrigger(boolean _trig);
	public abstract boolean queryTrigger();
}

class PollTrigger implements Runnable {
	private Event event;
	private AcceptEventAction parent;
	private boolean runOnce = false;	//Flag to indicate, the polling should take place once (For TimeEvents) or infinite.
	
	//Constructor which is called when a new PollTrigger is created which is sensitive to a TimeEvent
	public PollTrigger(TimeEvent _tim, AcceptEventAction _parent){
		event = _tim;
		this.parent = _parent;
		runOnce = true;
	}
	
	//Constructor which is called when a new PollTrigger is created which is sensitive to a ReceiveSignalEvent
	public PollTrigger(ReceiveSignalEvent rec, AcceptEventAction _parent){
		event = rec;
		this.parent = _parent;
	} 

//TODO This section should be reworked to be generated more intelligently 
//And also the Polling of an Event is not really nice, but it works which is enough for the moment! 
	public void run(){
		//Here, polling takes place for the corresponding event to occur
		if(runOnce){
			event.queryTrigger();	//This is the case for TimeEvent. This call returns, when the timeToWait is over 
			parent.letRun();	//Then, release the stop flag of the AEA-parent
		}
		else {				//This branch is taken, if event is sensititve to a signal. The polling is infinite
			while(true){
				if(!event.queryTrigger()){
					try{
						Thread.sleep(100);
					} catch (InterruptedException e){}
				}
				else {
					event.setTrigger(false);	//Reset the trigger of the Event. 
					parent.letRun();		//Release stop-flag of parent and let it run
				}
			}
		}
	}
	
	public String toString(){
		return event.getId();
	}
}


//Can be one of the attributes of trigger, which itself is an attribute of AcceptEventAction
class TimeEvent extends Event{
	private int timeToSleep;
	
	public TimeEvent(int timeTS){
		timeToSleep = timeTS;
		id = "TimeEvent";
	}
	
	public void setTimetoSleep(int time){
		timeToSleep = time;
	}
	
	public int getTimeToSleep(){
		return timeToSleep;
	}
	
	public void setTrigger(boolean trig){
		//Nothing here, the actual functionality is in the queryTrigger method
	}
	
	public boolean queryTrigger(){
		//"Catch" the polling Thread so that it waits 
		try{
			Thread.sleep(timeToSleep*1000);
		} catch (InterruptedException e) {}

		return true;
	}
	
	public String toString(){
		return timeToSleep + "s";
	}
}

//Can be another attribute of trigger, which itself is an attribute of AcceptEventAction
class ReceiveSignalEvent extends Event{
	AcSignal triggerSignal;
	volatile boolean sent = false;	//Indicates, if the signal was sent, on which this Event is sensitive to
	
	public ReceiveSignalEvent(AcSignal sig){
		triggerSignal = sig;
		id = "ReceiveSignalEvent";
		triggerSignal.registerEvent(this);
	}
	
	public void triggerSignal(AcSignal _trig){
		triggerSignal = _trig;
	}
	
	public AcSignal getTriggerSignal(){
		return triggerSignal;
	}
	
	public String getId(){
		return "Signal " + triggerSignal.getName();
	}
	
	//Method not synchronized, although multiple threads may access it. But, the sent variable is declared volatile
	public void setTrigger(boolean _s){
		sent = _s;
		
		if(_s)
			System.out.println("Signal " + triggerSignal.getName() + " was received");
	}
	
	public boolean queryTrigger(){
		return sent;
	}
}

